{% load i18n %}

<div role="tabpanel" class="tab-pane" id="tab-alliances">
    <div class="table-responsive">
        <table id="tbl-alliance-standings" class="table table-striped table-width-fix">
            <thead>
                <tr>
                    <th class="col-md-6">{% translate "Name" %}</th>
                    <th class="col-md-1">{% translate "Standing" %}</th>
                    <th class="col-md-5">{% translate "Labels" %}</th>
                </tr>
            </thead>
            <tbody>
            </tbody>
        </table>
    </div>
</div>
