# list of entity type IDs used in unit tests

CHARACTER_AMARR_TYPE_ID = 1373
CHARACTER_NI_KUNNI_TYPE_ID = 1374
CHARACTER_CIVRE_TYPE_ID = 1375
CHARACTER_DETEIS_TYPE_ID = 1376
CHARACTER_GALLENTE_TYPE_ID = 1377
CHARACTER_INTAKI_TYPE_ID = 1378
CHARACTER_SEBIESTOR_TYPE_ID = 1379
CHARACTER_BRUTOR_TYPE_ID = 1380
CHARACTER_STATIC_TYPE_ID = 1381
CHARACTER_MODIFIER_TYPE_ID = 1382
CHARACTER_ACHURA_TYPE_ID = 1383
CHARACTER_JIN_MEI_TYPE_ID = 1384
CHARACTER_KHANID_TYPE_ID = 1385
CHARACTER_VHEROKIOR_TYPE_ID = 1386
CHARACTER_DRIFTER_TYPE_ID = 34574

CHARACTER_TYPE_ID = CHARACTER_AMARR_TYPE_ID

CORPORATION_TYPE_ID = 2

ALLIANCE_TYPE_ID = 16159

FACTION_CALDARI_STATE_TYPE_ID = 500001

FACTION_TYPE_ID = FACTION_CALDARI_STATE_TYPE_ID
